package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;
import java.util.*;

// Spelchan Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.sprite.*;
import com.blazinggames.urcl.ui.comp.*;

// Retro Imports
import com.blazinggames.retro.shared.search.ThreeDPoint;

/**
 * Ultimate Retro Project Episode 6: Modern Nuclear Terrorism
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 * 
 * The modern version of Nuclear Terrorism
 * 
 * --------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.00.021025
 */
public class NukeDisplay 
extends Display
{
// *********************
// ***** CONSTANTS *****
// *********************

    private static final Point DEF_PLACE = new Point (0,0);
    private static final Rectangle DEF_RECT = new Rectangle(0,0,0,0);

    // Assets
    private static final String BACKDROP = "06BACKDROP";
    private static final String WIN_IMG = "06WIN";
    private static final String LOSE_IMG = "06LOSE";
    private static final String FRAMESIZE = "06FRAMESIZE";
    private static final String NUMFRAMES = "06NUMFRAMES";
    private static final String ANIMRECT = "06ANIMRECT";
    private static final String RESULTRECT = "06RESULTRECT";
    private static final String BOMBATRECT = "06BOMBWASATRECT";

    // Layout placements
    private static final String PLAYFIELD_RECT = "06PLAYFIELD";
    private static final String GEIGER_RECT = "06GEIGER";
    private static final String GEIGER_VALUE_POS = "06GEIGERVALUE";
    private static final String LAST_GUESS_POS = "06LASTGUESS";
    private static final String TIME_REMAINING_POS = "06REMAINING";
    private static final String CURRENT_FLOOR_POS = "06CURFLOOR";
    private static final String TOTAL_FLOORS_POS = "06TOTALFLOOR";
    private static final String FLOOR_SELECT_RECT = "06FLOORSELECT";

    // Levels
    private static final String NUM_LEVELS = "06NUM_LEVELS";
    private static final String LEVEL = "06LEVEL";
    private static final String BUILDING_FAR_RECTS = "06BUILDING_FAR";
    private static final String BUILDING_CLOSE_RECTS = "06BUILDING_CLOSE";

    // Geiger Counter
//    private static final String NUM_GEIGER_KEYS = "06NUM_GEIGER_KEYS";
//    private static final String GEIGER_KEY = "06GEIGER_KEY";
    private static final String RED_RANGE = "06REDRANGE";
    private static final String GREEN_RANGE = "06GREENRANGE";
    private static final String BLUE_RANGE = "06BLUERANGE";
    private static final String SPANS = "06SPANS";
    
    // Sound support
    private static final String USE_SOUND = "06USESOUND";
    private static final String SOUND_IMAGE = "06SOUNDIMAGE";
    private static final String SOUND_RECT = "06SOUNDRECT";
    private static final String CLICK_SOUND = "06CLICKSOUND";
    private static final String WIN_SOUND = "06WINSOUND";
    private static final String LOSE_SOUND = "06LOSESOUND";

    private static final String ELEVATOR_MESSAGE = "Please select floor";

    // Floor panel button array values - row 1
    private static final int FB_1 = 0;
    private static final int FB_2 = 1;
    private static final int FB_3 = 2;
    // Floor panel button array values - row 2
    private static final int FB_4 = 3;
    private static final int FB_5 = 4;
    private static final int FB_6 = 5;
    // Floor panel button array values - row 3
    private static final int FB_7 = 6;
    private static final int FB_8 = 7;
    private static final int FB_9 = 8;
    // Floor panel button array values - row 4
    private static final int FB_0 = 9;
    private static final int FB_CLEAR = 10;
    // Floor panel button array values - row 5
    private static final int FB_GO = 11;

// *********************
// ***** VARIABLES *****
// *********************

    private java.util.Random _random;

    // Assets
    private Image _background;
    private Image _winAnim;
    private Image _loseAnim;
    private Dimension _frameSize;
    private int _numFrames;
    private SLabel _results;
    private SLabel _bombAt;
    private Rectangle _animRect;
    private Image _curAnimation;
    private int _curFrame;
    private boolean _animInProgress;
    
    // Layout
    private Rectangle _playfieldRect;
    private Rectangle _geigerRect;
    private Rectangle _floorPanel;
    private Rectangle _highlight;
    private Font _historyFont;
    
    // Layout - text (sprites)
    private TextSprite _lastGuess;
    private TextSprite _geigerValue;
    private TextSprite _timeRemaining;
    private SLabel _currentFloor;
    private SLabel _totalFloors;
    
    // Floor selection Button panel
    private TextSprite _panelDisplay;
    private TextButton _panelButtons[];
    private int _curFloorValue;
    private int _newFloorValue;

    // Level Info
    private ThreeDPoint _levelSizes[];
    private int _levelTimes[];
    private Rectangle _buildingFarRects[];
    private Rectangle _buildingCloseRects[];
    
    // gameplay
    private int _level;
    private ThreeDPoint _bomb;
    private int _currentTime;
    private Dimension _cellSize;
    private boolean _downInPlayfield;
    private boolean _gameOver;

    // Gameplay - history
    private double _maxDistance;
    private double _lastGValue;
    private ThreeDPoint _historyPoints[];
    private Color _historyColor[];
    private int _historyValues[];
    private Color _geigerRange[];

    // Sound management
    boolean _useSound;
    Toggle _soundToggle;


// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    private void prepareAttributes()
    {
        _res.addImage(BACKDROP, _config.getStringProperty(BACKDROP, "backdrop.gif"));
        _background = _res.getImage(BACKDROP);

    	_playfieldRect = _config.getRectangleProperty(PLAYFIELD_RECT, DEF_RECT);
    	_geigerRect = _config.getRectangleProperty(GEIGER_RECT, DEF_RECT);
    	_floorPanel = _config.getRectangleProperty(FLOOR_SELECT_RECT, DEF_RECT);
        _highlight = new Rectangle(-1,-1, 0,0);
    }
    
    private void prepareSprites()
    {
        Container c = _displayHandler.getContainer();

        _res.addImage(WIN_IMG, _config.getStringProperty(WIN_IMG, "win.gif"));
        _winAnim = _res.getImage(WIN_IMG);
        _res.addImage(LOSE_IMG, _config.getStringProperty(LOSE_IMG, "lose.gif"));
        _loseAnim = _res.getImage(LOSE_IMG);
        Point p = _config.getPointProperty(FRAMESIZE, DEF_PLACE);
        _frameSize = new Dimension(p.x, p.y);
        _numFrames = _config.getIntProperty(NUMFRAMES, 0);
        
        _results = new SLabel(c, RESULTRECT, _config.getRectangleProperty(RESULTRECT, DEF_RECT));
        _results.setAlignment(SLabel.ALIGN_CENTER);
        _bombAt = new SLabel(c, BOMBATRECT, _config.getRectangleProperty(BOMBATRECT, DEF_RECT));
        _bombAt.setAlignment(SLabel.ALIGN_CENTER);
        _animRect = _config.getRectangleProperty(ANIMRECT, DEF_RECT);

        p = _config.getPointProperty(LAST_GUESS_POS, DEF_PLACE);
    	_lastGuess = new TextSprite(c, "last guess", p.x, p.y);
        _lastGuess.setVisible(true);
        
        p = _config.getPointProperty(GEIGER_VALUE_POS, DEF_PLACE);
        _geigerValue = new TextSprite(c, "geiger", p.x, p.y);
        _geigerValue.setVisible(true);
        
        p = _config.getPointProperty(TIME_REMAINING_POS, DEF_PLACE);
    	_timeRemaining = new TextSprite(c, "time", p.x, p.y);
        _timeRemaining.setVisible(true);
        
        Rectangle r = _config.getRectangleProperty(CURRENT_FLOOR_POS, DEF_RECT);
    	_currentFloor = new SLabel(c, "floor", r);
        _currentFloor.setAlignment(SLabel.ALIGN_CENTER);
        _currentFloor.setFont(c, new Font("Serif", 1, r.height - 4));
//        _currentFloor.setBackground(Color.blue);
        
        r = _config.getRectangleProperty(TOTAL_FLOORS_POS, DEF_RECT);
    	_totalFloors = new SLabel(c, "totalFloors", r);
        _totalFloors.setAlignment(SLabel.ALIGN_CENTER);
        _totalFloors.setFont(c, new Font("Serif", 0, r.height - 4));
//        _totalFloors.setBackground(Color.blue);
        prepareFloorPanel();
    }
    
    /**
     * This sets up all the buttons and the sprite used in displaying
     * the floor selection panel.
     */
    private void prepareFloorPanel()
    {
	Container c = _displayHandler.getContainer();
	_panelDisplay = new TextSprite(c, ELEVATOR_MESSAGE, _floorPanel.x, _floorPanel.y);
	_panelDisplay.setVisible(true);
	_panelButtons = new TextButton[12];
	
	int bw = _floorPanel.width / 3 - 2;
	int bh = _floorPanel.height / 6 - 2;
	int closex = _floorPanel.x + 1;
	int midx = _floorPanel.x + bw + 3;
	int farx = midx + bw + 3;
	int row = _floorPanel.y + bh + 3;

	_panelButtons[FB_1] = new TextButton(c, "1", closex, row, bw, bh);
	_panelButtons[FB_2] = new TextButton(c, "2", midx, row, bw, bh);
	_panelButtons[FB_3] = new TextButton(c, "3", farx, row, bw, bh);

	row = row + bh + 3;
	_panelButtons[FB_4] = new TextButton(c, "4", closex, row, bw, bh);
	_panelButtons[FB_5] = new TextButton(c, "5", midx, row, bw, bh);
	_panelButtons[FB_6] = new TextButton(c, "6", farx, row, bw, bh);

	row = row + bh + 3;
	_panelButtons[FB_7] = new TextButton(c, "7", closex, row, bw, bh);
	_panelButtons[FB_8] = new TextButton(c, "8", midx, row, bw, bh);
	_panelButtons[FB_9] = new TextButton(c, "9", farx, row, bw, bh);

	row = row + bh + 3;
	_panelButtons[FB_0] = new TextButton(c, "0", closex, row, bw, bh);
	_panelButtons[FB_CLEAR] = new TextButton(c, "CLEAR", midx, row, bw + bw + 2, bh);

	row = row + bh + 3;
	_panelButtons[FB_GO] = new TextButton(c, "GO TO FLOOR", closex, row, bw * 3 + 4, bh);
    }

    private void prepareSounds()
    {
        _useSound = (_config.getIntProperty(USE_SOUND, 0) == 1);
        _res.addImage(SOUND_IMAGE, _config.getStringProperty(SOUND_IMAGE, "sound.gif"));
        Rectangle r = _config.getRectangleProperty(SOUND_RECT, DEF_RECT);
    	_soundToggle = new Toggle(_displayHandler.getContainer(),
                    _res.getImage(SOUND_IMAGE), r.x, r.y, r.width, r.height, 2);
        if (_useSound)
        {
            _res.addAudioClip(CLICK_SOUND, _config.getStringProperty(CLICK_SOUND, "silent.au"));
            _res.addAudioClip(WIN_SOUND, _config.getStringProperty(WIN_SOUND, "silent.au"));
            _res.addAudioClip(LOSE_SOUND, _config.getStringProperty(LOSE_SOUND, "silent.au"));
        }
    }

    private void prepareLevels()
    {
        int n = _config.getIntProperty(NUM_LEVELS, 0);
        if (n < 1)
            n = 1;

System.out.println("There are " + n + " levels.");

        _levelSizes = new ThreeDPoint[n];
        _levelTimes = new int[n];
        _buildingFarRects = new Rectangle[n];
        _buildingCloseRects = new Rectangle[n];
        String args[];
        
        for (int cntr = 0; cntr < n; ++cntr)
        {
            args = _config.parseString(_config.getStringProperty(LEVEL + "[" + cntr + "]", "x"), ',');
            if (args.length != 4)
            {
                _levelSizes[cntr] = new ThreeDPoint(10,10,10);
                _levelTimes[cntr] = 30;
            }
            else
            {
                _levelSizes[cntr] = new ThreeDPoint(
                            _config.parseInt(args[0], 10),
                            _config.parseInt(args[1], 10),
                            _config.parseInt(args[2], 10) );
                _levelTimes[cntr] = _config.parseInt(args[3], 30);
            }

            _buildingFarRects[cntr] = _config.getRectangleProperty(BUILDING_FAR_RECTS + "[" + cntr + "]", DEF_RECT);
            _buildingCloseRects[cntr] = _config.getRectangleProperty(BUILDING_CLOSE_RECTS + "[" + cntr + "]", DEF_RECT);
        }
        prepareGeigerRange();
    }

    private void prepareGeigerRange()
    {
        String reds[] = _config.parseString(_config.getStringProperty(RED_RANGE, "0"), ',');
        String greens[] = _config.parseString(_config.getStringProperty(GREEN_RANGE, "0"), ',');
        String blues[] = _config.parseString(_config.getStringProperty(BLUE_RANGE, "0"), ',');
        String spansStr[] = _config.parseString(_config.getStringProperty(SPANS, "0"), ',');

        int temp = reds.length;
        int redRange[] = new int[temp];
        int greenRange[] = new int[temp];
        int blueRange[] = new int[temp];
        int spans[] = new int[temp];

        int totalSpan = 0;
        int cntr;
        int r, g, b, rs, gs, bs, spot = 0;

        r = g = b = rs = 10;
        for (cntr = 0; cntr < temp; ++cntr)
        {
            try
            {
                r = _config.parseInt(reds[cntr], 0);
                g = _config.parseInt(greens[cntr], 0);
                b = _config.parseInt(blues[cntr] , 0);
                if (cntr == (temp - 1))
                    rs = 0;
                else
                    rs = _config.parseInt(spansStr[cntr], 1);
            }
            catch (ArrayIndexOutOfBoundsException ignore) {System.err.println("Array out of bounds in Geiger");}
            
            redRange[cntr] = r;
            greenRange[cntr] = g;
            blueRange[cntr] = b;
            spans[cntr] = rs;
            totalSpan += rs;
        }
        
        _geigerRange = new Color[totalSpan];
        for (cntr = 0; cntr < (temp - 1); ++cntr)
        {
            rs = redRange[cntr + 1] - redRange[cntr];
            gs = greenRange[cntr + 1] - greenRange[cntr];
            bs = blueRange[cntr + 1] - blueRange[cntr];
            for (int cntrSpan = 0; cntrSpan < spans[cntr]; ++cntrSpan)
            {
                r = redRange[cntr] + (rs * cntrSpan) / spans[cntr];
                g = greenRange[cntr] + (gs * cntrSpan) / spans[cntr];
                b = blueRange[cntr] + (bs * cntrSpan) / spans[cntr];
                _geigerRange[spot] = new Color(r, g, b);
                ++spot;
            }
        }
    }



// ***********************************
// ***** METHODS -  From Display *****
// ***********************************

    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);

    	_background = null;
        _random = new java.util.Random();
        dh.displayEvent(this, "LOADING 25");

        prepareAttributes();
        dh.displayEvent(this, "LOADING 50");
        prepareSprites();
        dh.displayEvent(this, "LOADING 75");
        prepareSounds();
        prepareLevels();
        dh.displayEvent(this, "LOADING 100");
                
        return true;
    }

    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = _soundToggle.mouse_move(x, y, b);
    
        // floor select button handling
        for (int cntr = 0; cntr < 12; ++ cntr)
        {
            repaint |= _panelButtons[cntr].mouse_move(x, y, b);
        }

        if ( (! _gameOver) && (_playfieldRect.contains(x, y)) )
        {
            if (_highlight.contains(x, y) == false)
            {
                int tilex = (x - _playfieldRect.x) / _cellSize.width;
                int tiley = (y - _playfieldRect.y) / _cellSize.height;
                _highlight.setLocation(_playfieldRect.x + tilex * _cellSize.width, 
                        _playfieldRect.y + tiley * _cellSize.height);
                repaint = true;
            }
        }
        else if (_highlight.x != -1)
        {
            _highlight.setLocation(-1, -1);
            repaint = true;
        }
    
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    public void mouse_down(int x, int y)
    {
        boolean repaint = _soundToggle.mouse_down(x, y);
        _downInPlayfield = _playfieldRect.contains(x, y);

        // floor select button handling
        for (int cntr = 0; cntr < 12; ++ cntr)
        {
            repaint |= _panelButtons[cntr].mouse_down(x, y);
        }
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    public void mouse_up(int x, int y)
    {
        boolean repaint = _soundToggle.mouse_up(x, y);
    
        if (_gameOver)
        {
            _displayHandler.displayEvent(this, "NUKE_TITLE");
        }
        else if ( (_downInPlayfield) && (_playfieldRect.contains(x, y)) )
        {
            int guessx = (x - _playfieldRect.x) / _cellSize.width + 1;
            int guessy = (y - _playfieldRect.y) / _cellSize.height + 1;
            _displayHandler.log("player guesses " + guessx + "," + guessy + "," + _curFloorValue);
            _historyPoints[_currentTime] = new ThreeDPoint(guessx, guessy, _curFloorValue);
            double dpct = _historyPoints[_currentTime].getDistance(_bomb) / _maxDistance * 100;
            dpct = (double)((int)(dpct * 100)) / 100.0;
            _lastGValue = 100.0 - dpct;
            _historyValues[_currentTime] = (int)_lastGValue;
            _historyColor[_currentTime] = _geigerRange[(int)_lastGValue * (_geigerRange.length - 1) / 100];
            if (_historyPoints[_currentTime].equals(_bomb))
            {
                setWin();
            }
            else if (_currentTime == (_levelTimes[_level] - 1))
            {
                setLose();
            }
            else
            {
                playSound(CLICK_SOUND);
            }
            
            ++_currentTime;
            updateSprites();
            repaint = true;
        }
    
        // floor select button handling
        for (int cntr = 0; cntr < 12; ++ cntr)
        {
            if (_panelButtons[cntr].mouse_up(x, y))
            {
            handleFloorButton(cntr);
            repaint = true;
            }
        }
    
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    private void setWin()
    {
        Container c = _displayHandler.getContainer();
        
        _results.setText(c, "WELL DONE!");
        _bombAt.setText(c, "You found the bomb!");
    	_curAnimation = _winAnim;
    	_curFrame = 0;
    	_animInProgress = true;
        _gameOver = true;
        playSound(WIN_SOUND);
    }
    
    private void setLose()
    {
        Container c = _displayHandler.getContainer();
        
        _results.setText(c, "Good thing this is a game!");
        _bombAt.setText(c, "Bomb was at " + _bomb.x + "," + _bomb.y + " floor " + _bomb.z );
    	_curAnimation = _loseAnim;
    	_curFrame = 0;
    	_animInProgress = true;
        _gameOver = true;
        playSound(LOSE_SOUND);
    }
    
    public void tick(int ms)
    {
        boolean repaint = _animInProgress;
    
        // floor select button handling
        for (int cntr = 0; cntr < 12; ++ cntr)
        {
            repaint |= _panelButtons[cntr].tick(ms);
        }
    
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

    public synchronized void render(Graphics g, Component comp)
    {
        if (_background == null)
        {
            Dimension d = comp.getSize();
            g.setColor(Color.white);
            g.fillRect(0,0, d.width, d.height);
        }
        else
        {
            g.drawImage(_background, 0, 0, comp);
        }

        renderGeiger(g, _geigerRect, (int)_lastGValue);
        renderHistory(g);
        renderSprites(g, comp);
    
        for (int cntr = 0; cntr < 12; ++cntr)
        {
            _panelButtons[cntr].render(g, comp);
        }
        _panelDisplay.render(g, comp);
        renderFloor(g);
        _soundToggle.render(g, comp);

        if (_gameOver)
            renderOver(g, comp);
    }


// ************************************
// ***** METHODS - Rendering aids *****
// ************************************
    
    private void renderSprites (Graphics g, Component c)
    {
    	_lastGuess.render(g, c);
    	_geigerValue.render(g, c);
    	_timeRemaining.render(g, c);
    	_currentFloor.render(g, c);
    	_totalFloors.render(g, c);
    }

    private void renderFloor(Graphics g)
    {
        int cntr;
    
        g.setFont(_historyFont);
        for (cntr = 0; cntr < _currentTime; ++cntr) 
        {
            if (_historyPoints[cntr].z == _curFloorValue) 
            {
                g.setColor(_historyColor[cntr]);
                g.fillRect(_playfieldRect.x + (_historyPoints[cntr].x - 1) * _cellSize.width,
                    _playfieldRect.y + (_historyPoints[cntr].y - 1) * _cellSize.height,
                    _cellSize.width, _cellSize.height);
                g.setColor(Color.black);
                g.drawString("" + _historyValues[cntr],
                    _playfieldRect.x + (_historyPoints[cntr].x - 1) * _cellSize.width + 2, 
                    _playfieldRect.y + (_historyPoints[cntr].y) * _cellSize.height - 2);
            }
        }

        g.setColor(Color.blue);
        for (cntr = 1; cntr < _levelSizes[_level].x; ++cntr)
        {
            g.drawLine(_playfieldRect.x + cntr * _cellSize.width,
                _playfieldRect.y,
                _playfieldRect.x + cntr * _cellSize.width,
                _playfieldRect.y + _playfieldRect.height);
        } 

        for (cntr = 1; cntr < _levelSizes[_level].y; ++cntr)
        {
            g.drawLine(_playfieldRect.x,
                _playfieldRect.y + cntr * _cellSize.height,
                _playfieldRect.x + _playfieldRect.width,
                _playfieldRect.y + cntr * _cellSize.height );
        }
        
        if (_highlight.x >= 0)
        {
            g.setColor(Color.yellow);
            g.drawRect(_highlight.x, _highlight.y, _highlight.width, _highlight.height);
        }
    }

    private void renderGeiger(Graphics g, Rectangle place, int percent)
    {
        if (place.width < 1) return;
        int units = place.width * percent / 100;
        int cindex;
    
        g.setColor(Color.black);
        g.fillRect(place.x + units, place.y, place.width - units, place.height);
        for (int cntr = 0; cntr < units; ++cntr)
        {
            cindex = cntr * _geigerRange.length / place.width;
            g.setColor(_geigerRange[cindex]);
            g.drawLine(place.x + cntr, place.y, place.x + cntr, place.y + place.height);
        }
    }

    private void renderHistory(Graphics g)
    {
        Rectangle fr = _buildingFarRects[_level];
        Rectangle cr = _buildingCloseRects[_level];
    
        g.setColor(Color.white);
        g.drawRect(fr.x, fr.y, fr.width, fr.height);
    
        // draw history list here
        ThreeDPoint tdp = _levelSizes[_level];
        int w = Math.max(1, fr.width / tdp.x);
        int h = Math.max(1, fr.height / tdp.y);
        int distx = cr.x - fr.x;
        int disty = cr.y - fr.y;
        int relx, rely, x, y;
        for (int cntr = 0; cntr < _currentTime; ++cntr) {
            relx = fr.width * _historyPoints[cntr].x / tdp.x;
            rely = fr.height * (tdp.z - _historyPoints[cntr].z) / tdp.z;
            x = distx * _historyPoints[cntr].y / tdp.y;
            y = disty * _historyPoints[cntr].y / tdp.y;
            x = x + relx + fr.x;
            y = y + rely + fr.y;
            g.setColor(_historyColor[cntr]);
            g.fillRect(x, y, w, h);
        }
    
        g.setColor(Color.white);
        g.drawLine(fr.x, fr.y, cr.x, cr.y);
        g.drawLine(fr.x + fr.width, fr.y, cr.x + cr.width, cr.y);
        g.drawLine(fr.x, fr.y + fr.height, cr.x, cr.y + cr.height);
        g.drawLine(fr.x + fr.width, fr.y + fr.height, cr.x + cr.width, cr.y + cr.height);
        g.drawRect(cr.x, cr.y, cr.width, cr.height);
    }

    private void renderOver(Graphics g, Component c)
    {
        _results.render(g, c);

    	g.drawImage(_curAnimation,
                    _animRect.x, _animRect.y, 
                    _animRect.x + _animRect.width, _animRect.y + _animRect.height,
                    _curFrame % 8 * 128, _curFrame / 8 * 128,
                    _curFrame % 8 * 128 + 128, _curFrame / 8 * 128 + 128,
                    c);
        
        if (_animInProgress)
        {
            ++_curFrame;
            if (_curFrame > 23)
            {
                _curFrame = 23;
                _animInProgress = false;
            }
        }

        _bombAt.render(g, c);
    }


// *****************************************
// ***** MEHTODS - Gameplay management *****
// *****************************************

    private int rand(int n)
    {
        return (((_random.nextInt() & Integer.MAX_VALUE) % n) + 1);
    }
    
    public void setLevel(int n)
    {
        _level = n;
        
        _displayHandler.log("Level has been set to " + n);
        startNewGame();
    }
    
    private void startNewGame()
    {
        int bombX = rand(_levelSizes[_level].x);
        int bombY = rand(_levelSizes[_level].y);
        int bombZ = rand(_levelSizes[_level].z);
        _lastGValue = 100;
        _maxDistance = ThreeDPoint.calculateDistance(0,0,0, 
                                _levelSizes[_level].x,
                                _levelSizes[_level].y,
                                _levelSizes[_level].z );
        _historyPoints = new ThreeDPoint[_levelTimes[_level]];
        _historyColor = new Color[_levelTimes[_level]];
        _historyValues = new int[_levelTimes[_level]];
        _displayHandler.log("Bomb at " + bombX + "," + bombY + "," + bombZ);
        _bomb = new ThreeDPoint(bombX, bombY, bombZ);
        _currentTime = 0;
        _cellSize = new Dimension(_playfieldRect.width / _levelSizes[_level].x,
                    _playfieldRect.height / _levelSizes[_level].y);
        _historyFont = new Font("Serif", 0, _cellSize.height/2);
        _highlight.setBounds(-1, -1, _cellSize.width, _cellSize.height);
        _curFloorValue = 1;
        _newFloorValue = 0;
        _gameOver = false;
        updateSprites();
    }

    private void updateSprites()
    {
        Container c = _displayHandler.getContainer();
        _currentFloor.setText(c, "" + _curFloorValue);
        _timeRemaining.setText(c, "" + (_levelTimes[_level] - _currentTime) + ":00");
        _totalFloors.setText(c, "" + _levelSizes[_level].z);

        if (_currentTime == 0) 
        {
            _lastGuess.setText(c, "none");
            _geigerValue.setText(c, "NA");
        }
        else 
        {
            int decimal = (int)((_lastGValue * 100) % 100);
            int percent = (int)((_lastGValue * 100) / 100);
            _geigerValue.setText(c, percent +"." + decimal);
            ThreeDPoint tdp = _historyPoints[_currentTime - 1];
            _lastGuess.setText(c, "" + tdp.x + "," + tdp.y + "," + tdp.z);
        }
    }

    private void handleFloorButton(int nID)
    {
	_displayHandler.log("Button id " + nID + " pressed.");
	if (nID < FB_0)
	{
	    _newFloorValue = _newFloorValue * 10 + nID + 1;
	}
	else if (nID == FB_0)
	{
	    _newFloorValue *= 10;
	}
	else if (nID == FB_CLEAR)
	{
	    _newFloorValue = 0;
	}
	else
	{
	    if ( (_newFloorValue > 0) && (_newFloorValue <= _levelSizes[_level].z) )
		{
		    _curFloorValue = _newFloorValue;
		}
	    else
		{
		    // *TODO* play beep sound
		}
	    _newFloorValue = 0;
	}

	Container c = _displayHandler.getContainer();
	if (_newFloorValue == 0)
	    {
		_panelDisplay.setText(c, ELEVATOR_MESSAGE);
	    }
	else
	    {
		_panelDisplay.setText(c, "" + _newFloorValue);
	    }
	updateSprites();
    }

    /**
     * If sound is enabled gets the resource manager to play the indicated sound.
     *
     * @param key sound to play
    */
    private void playSound(String key)
    {
        if ( (_useSound) && (_soundToggle.getState() == 1) )
        {
            _res.playAudio(key);
        }
    }
}
