package com.blazinggames.retro.shared.search;

/**
 * Ultimate Retro Project Episode 5 & 6: Classic and Modern Nuclear Terrorism
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 * 
 * A 3D point, with some methods that will be utilized in Nuclear Terrorism.
 * 
 * --------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.00.010918
*/
public class ThreeDPoint 
{
// *********************
// ***** VARIABLES *****
// *********************

    /** X location. Public for convienience. */
    public int x;
    /** Y location. Public for convienience. */
    public int y;
    /** Z location. Public for convienience. */
    public int z;
    

// ************************
// ***** CONSTRUCTORs *****
// ************************

    /**
     * Creates the default (0,0,0) point.
    */
    public ThreeDPoint()
    {
        x = 0;
        y = 0;
        z = 0;
    }
    
    /**
     * Creates the 3D point based on the provided ThreeDPoint class.
     *
     * @param tdp the class to be cloned
    */
    public ThreeDPoint(ThreeDPoint tdp)
    {
        x = tdp.x;
        y = tdp.y;
        z = tdp.z;
    }
    
    /**
     * Creates a 3D point based on the provided parameters.
     *
     * @param nx = X coordinate
     * @param ny = Y coordinate
     * @param nz = Z coordinate
    */
    public ThreeDPoint(int nx, int ny, int nz)
    {
        x = nx;
        y = ny;
        z = nz;
    }


// *****************************************
// ***** METHODS - Getting and Setting *****
// *****************************************

    /**
     * sets the coordinates to the indicated values.
     *
     * @param nx new x coordinate
     * @param ny new y coordinate
     * @param nz new z coordinate
    */
    public void setCoordinates(int nx, int ny, int nz)
    {
        x = nx;
        y = ny;
        z = nz;
    }
    
    /**
     * sets the X coordinate to the indicated value.
     *
     * @param n new x coordinate
    */
    public void setX(int n)
    {
        x = n;
    }
    
    /**
     * @return X coordinate
    */
    public int getX()
    {
        return x;
    }

    /**
     * sets the Y coordinate to the indicated value.
     *
     * @param n new y coordinate
    */
    public void setY(int n)
    {
        y = n;
    }
    
    /**
     * @return Y coordinate
    */
    public int getY()
    {
        return y;
    }
    
    /**
     * sets the Z coordinate to the indicated value.
     *
     * @param n new z coordinate
    */
    public void setZ(int n)
    {
        z = n;
    }
    
    /**
     * @return Z coordinate
    */
    public int getZ()
    {
        return z;
    }


// *********************************
// ***** METHODS - From Object *****
// *********************************

    public boolean equals(Object obj)
    {
        try
        {
            ThreeDPoint tdp = (ThreeDPoint)obj;
            if ( (tdp.x == x) && (tdp.y == y) && (tdp.z == z) )
                return true;
            else
                return false;
        }
        catch (Exception e)
        {
            return false;
        }
    }


// ***************************************************
// ***** METHODS - Nuclear Terrorism aid methods *****
// ***************************************************
    
    /** 
     * Finds the distance this 3D point 
     * is from the indicated point.
     *
     * @param nx X coordinate
     * @param ny Y coordinate
     * @param nz Z coordinate
     * @return calculated distance points are from each other
    */
    public double getDistance(int nx, int ny, int nz)
    {
        return calculateDistance(nx, ny, nz, x, y, z);
    }

    /** 
     * Finds the distance this 3D point 
     * is from the indicated point.
     *
     * @param tdp point to check against.
     * @return calculated distance points are from each other
    */
    public double getDistance(ThreeDPoint tdp)
    {
        return calculateDistance(tdp.x, tdp.y, tdp.z, x, y, z);
    }

    /** 
     * Finds the distance.
     *
     * @param x1 first X coordinate
     * @param y1 first Y coordinate
     * @param z1 first Z coordinate
     * @param x2 second X coordinate
     * @param x2 second X coordinate
     * @param x2 second X coordinate
     * @return calculated distance points are from each other
    */
    public static double calculateDistance(int x1, int y1, int z1, int x2, int y2, int z2)
    {
        int temp = x2 - x1;
        int sx = temp * temp;
        temp = y2 - y1;
        int sy = temp * temp;
        temp = z2 - z1;
        int sz = temp * temp;
        return Math.sqrt(sx + sy + sz);
    }
}
